require 'spec_helper'
require 'debci/not_found'
require 'rack/test'

describe Debci::NotFound do
  class NotFound < Debci::NotFound
    set :raise_errors, true
    set :show_exceptions, false

    def erb(template, *args, **opts)
      self.class.last_template = template unless template.to_s =~ /^_/
      super(template, *args, **opts)
    end

    class << self
      attr_accessor :last_template
    end
  end

  def app
    Rack::Builder.new do
      run NotFound
    end
  end

  include Rack::Test::Methods

  let(:theuser) { Debci::User.create!(username: 'debci') }

  def last_template
    NotFound.last_template
  end

  it 'render Not Found template on 404' do
    get '/bla'
    expect(last_template).to eq(:not_found)
  end

  it 'renders log expired template' do
    package = Debci::Package.create!(name: 'xpkg')
    job = package.jobs.create!(
      date: Time.now - 2.years,
      suite: 'unstable',
      arch: 'arm64',
      requestor: theuser,
    )
    get "/data/autopkgtest/unstable/arm64/x/xpkg/#{job.run_id}/log.gz"
    expect(last_template).to eq(:log_expired)
  end

  it 'renders normal 404 for unexisting jobs' do
    Debci::Package.create!(name: 'xpkg')
    get "/data/autopkgtest/unstable/arm64/x/xpkg/1237289371289/log.gz"
    expect(last_template).to eq(:not_found)
  end
end
